# Exclude Land Processes in NEMO

The **E**xclude **L**and **P**rocesses **i**n **N**EMO tool is a tool that allow to find proper namelist parameters to exclude Land-only processes in NEMO simulations.

To use the tool, just clone it to your work station and launch the ./Launcher and follow the instructions!

The first time that you run the tool it will install it. After that, with the resolution and the maximum number of processes to use, it will give you the 5 better domain decompositions to run with a maximum number of processors.


As an example, once the tool is installed, the command ``./Launcher ORCA025 512`` will produce :


```
Resolution  ORCA025
Max number of processors: 512


It will be better if you run with one of these configurations:buBigger coefficient is better!
jpij: 499 jpi: 37 jpj: 17  OriginalP: 629  LandP: 130 SeaP: 499  Coeficient: 672.25
jpij: 497 jpi: 30 jpj: 21  OriginalP: 630  LandP: 133 SeaP: 497  Coeficient: 670.73
jpij: 510 jpi: 34 jpj: 19  OriginalP: 646  LandP: 136 SeaP: 510  Coeficient: 669.28
jpij: 490 jpi: 25 jpj: 25  OriginalP: 625  LandP: 135 SeaP: 490  Coeficient: 664.58
jpij: 504 jpi: 29 jpj: 22  OriginalP: 638  LandP: 134 SeaP: 504  Coeficient: 662.45
```

It has two more optional arguments:
* **-s** to get a simplified output of only the best solution.
* **-l #** with # the number of elements to output.

so ``./Launcher ORCA025 512 -s`` will output: 

```
499 37 17

```

and ``./Launcher ORCA025 512 -l 3`` will produce:

```
Resolution  ORCA025
Max number of processors: 512


It will be better if you run with one of these configurations:buBigger coefficient is better!
jpij: 499 jpi: 37 jpj: 17  OriginalP: 629  LandP: 130 SeaP: 499  Coeficient: 672.25
jpij: 497 jpi: 30 jpj: 21  OriginalP: 630  LandP: 133 SeaP: 497  Coeficient: 670.73
jpij: 510 jpi: 34 jpj: 19  OriginalP: 646  LandP: 136 SeaP: 510  Coeficient: 669.28
```


The parameters in the first three columns indicate the values that must go to the *NEMO* **namelist**.
The others parameters indicate the following:
* **OriginalP** indicate the number of subdomains with a determined decomposition without removing the land processes.
* **LandP** indicate the number of only-land subdomains.
* **SeaP** indicate the number of subdomains that include at least one sea-point.
* The **Coefficient** value is computed using the number of processors (**P**), the size of the original grid (**Os**) and the sum of the size of all the sea-subdomains (**Fs**).
The formula used is the following: **Coef** = **P** · **Os**^2 / **Fs**^2. Then, this value increases when more processors are used but is modulated with the **Os**^2 / **Fs**^2 factor. This factor is mainly affected by two things:
 * More overlapping between subdomains decreases this factor.
 * Being able to remove more land-only processes increases this factor.
 
Using the numbers provided by this script allow to succefully run without the processes that do not contain any sea point.
## Example outputs
### ORCA12
In the next plot we can see the sea surface temperature of an ORCA12 simulation where the land-only processes have been removed (in white the removed parts):

![result](/uploads/14436bb19990e5ab7563134a40c950fe/RemovedProcesses.png)
### ORCA025 comparison
The next plots are both ORCA025 simulations where the land-only processes have been removed, the first one using 120 processes and the second one using 950 processes.
As can be seen, when more processes are used the size of the subdomains is smaller and therefore it is possible to adjust better to the continental shape and delete a bigger proportion of the land grid-points.

From domain decomposition resulting in 136 sub-domains 16 are land-only and therefore removed, an 11% of the total grid. 
![O025_120p](/uploads/775d2de2a2ade97dc9a54b56ff7e4ebf/RemovedProcessesORCA025_120.png)
From domain decomposition resulting in 1287 sub-domains 312 are land-only and therefore removed, a 24% of the total grid.
![O025_120p](/uploads/42cd6b018eefc525f9349b0d9d5101ef/RemovedProcessesORCA025_950.png)


## Impact on Performance

For the ORCA025 resolution, the advantadge of using ELPiN is massive as can be seen in the next figure:

![simple_plot](/uploads/898820c14fb18e569e372b2223d44f6d/simple_plot.png)


## Todo:

* Instead of using the MPP_PREP_TOOL included in NEMO, an alternative FORTRAN binary will be used to find optimal configurations faster.
* Try to integrate the same procedure inside NEMO itself.